//////////////////////////////////////////////////////////
//
// synthetic test scenes for
// structured light survey
// 
// christoph schmalz
// 2009
//
// christoph(a)structuredlightsurvey.de
//
//////////////////////////////////////////////////////////
//
// version 1.0 - first public release
// version 1.1 - added transformations
//
//////////////////////////////////////////////////////////
//
// defines a texture that color-codes the depth of a point
//
//////////////////////////////////////////////////////////

#macro depthCodingTexture(minDist,maxDist,posVector,dirVector)

	#local nDirVector = vnormalize(dirVector);

	//in order to clearly show the end of the range
	//we build in buffers, else the gradient just wraps around ...

	#local depthRange = maxDist-minDist;

	//the real range with buffers will be 3*depthRange
	//start at minZ-depthRange
	//end at maxZ+depthRange

	//the color map can only have 256 entries
	//so we cannot use all 16.7 million colors

	#macro redLevel(n)
		(n+1)*20/255
	#end

	#macro greenLevel(n)
		(n+1)*20/255
	#end
	
	#macro gradientLevel(n,m,nmax,mmax)
		1/3 + n*(1/3/nmax) + m*(1/3/nmax/mmax)
	#end

	#local redSteps = 10;
	#local greenSteps = 10;

	//this gives us 256*100 quantization steps

	material{
	    texture{
	     pigment{ 
	     	gradient nDirVector
	     	//map everything below 1/3 to white
	     	//map everything above 2/3 to white
	     	color_map {
		       [ 1/3 rgb 1 ] //out of range
				#local redStep = 0;
				#while(redStep < redSteps)
					#local greenStep = 0;
					#while(greenStep < greenSteps)

						//#debug concat( str(gradientLevel(redStep,greenStep,redSteps,greenSteps),0,5), " --> ", vstr(3,<redLevel(redStep),greenLevel(greenStep),0>,",",0,3), "\n" )
						//#debug concat( str(gradientLevel(redStep,greenStep+1,redSteps,greenSteps),0,5), " --> ", vstr(3,<redLevel(redStep),greenLevel(greenStep),1>,",",0,3), "\n" )

				       [ gradientLevel(redStep,greenStep,redSteps,greenSteps) rgb <redLevel(redStep),greenLevel(greenStep),0> ]
				       [ gradientLevel(redStep,greenStep+1,redSteps,greenSteps) rgb <redLevel(redStep),greenLevel(greenStep),1> ]

					  #local greenStep = greenStep + 1;
					#end
				  #local redStep = redStep + 1;
				#end
		       [ 2/3 rgb 1 ] //out of range
	     	}
	     }
	     finish{
	         diffuse 0
	         ambient 1
	     }


		translate -nDirVector/3	//move 'real' texture to begin at 0
		scale 3					//scale it to end at 1
		scale depthRange		//make length depthRange
		translate minDist*nDirVector //move it to begin at minDist
		translate posVector		//relative to the given position
	 }
	}

#end