% //////////////////////////////////////////////////////////
% //
% // synthetic test scenes for
% // structured light survey
% // 
% // christoph schmalz
% // 2009
% //
% // christoph(a)structuredlightsurvey.de
% //
% //////////////////////////////////////////////////////////
% //
% // version 1.0 - first public release
% //
% //////////////////////////////////////////////////////////
% //
% //	tries to compare all .xyz files in datadir with their counterpart in groundtruthdir
% //	to that end the filename is split at the first underscore
% //	ie datadir/testscenes2_noisy1_run2.xyz -> groundtruthdir/testscenes2.xyz
% //
% //	writes results to datadir/comparison.txt
% //
% //////////////////////////////////////////////////////////

function results = compareDepthmaps( datadir, groundtruthdir )

%declare important constants
maxDeviation = 10; %mm from ground truth, over limit means outlier
maxTriangleArea = 20; %mm^2 large triangles are bad for interpolation
maxTriangleFaceLength = 10; %mm long faces mostly indicate a jump in height
                            %and are also bad for interpolation

verbose = 0;
                            
results = struct;
files = dir(datadir);

outfile = fopen( fullfile(datadir,'comparison.txt'), 'w' );
fprintf(outfile,'name\toutliers\tinliers\terror mean\terror sigma\n','char');

for f = 1:size(files,1)
    dataname = fullfile(datadir, files(f).name );

    if isdir(dataname)
        continue;
    end

    [pathstr, name, ext, versn] = fileparts(dataname);

    if strcmp(ext,'.xyz')
        underscore_index = findstr(name,'_');

        if ~isempty(underscore_index)
            gtname = fullfile(groundtruthdir, strcat( name(1:underscore_index(1)-1), ext ) );
        else
            gtname = fullfile(groundtruthdir, files(f).name );
        end

        disp(sprintf('Comparing %s and %s',dataname,gtname));
        results.(name) = compareDepthMapsXYZ( dataname, gtname, 1, maxDeviation, ...
                                                maxTriangleArea, maxTriangleFaceLength, 3, verbose );

        str = sprintf('%s\t%d\t%d\t%d\t%d\n',name, ...
            results.(name).outliers, results.(name).inliers, ...
            results.(name).errorMean, results.(name).errorSigma );
        fprintf(outfile,str,'char');
    end

    if strcmp(ext,'.bmp') || strcmp(ext,'.png') || strcmp(ext,'.tif')
        underscore_index = findstr(name,'_');

        if ~isempty(underscore_index)
            gtname = fullfile(groundtruthdir, strcat( name(1:underscore_index(1)-1), '.bmp' ) );
        else
            gtname = fullfile(groundtruthdir, files(f).name, '.bmp' );
        end

        disp(sprintf('Comparing %s and %s',dataname,gtname));

        %load data
        gt = decodeDepthmap(imread(gtname));
        m = decodeMV3DCamDepthmap(imread(dataname));
        diff = m-gt;

        if verbose
            figure;
            imagesc(diff,[-maxDeviation maxDeviation]);
            colorbar;
            title(name);
        end
        
        %check for outliers
        result.outliers = sum (sum ( abs(diff)>maxDeviation ) );

        %set outliers to NaN
        diff ( abs(diff)>maxDeviation ) = NaN;

        %calculate statistics
        result.inliers = numel(diff) - sum(sum( isnan(diff) ));
        result.errorMean = nanmean(nanmean(diff));
        result.errorSigma = nanstd(reshape(diff,1,[]));
        
        str = sprintf('%s\t%d\t%d\t%d\t%d\n',name, ...
            result.outliers, result.inliers, ...
            result.errorMean, result.errorSigma );
        fprintf(outfile,str,'char');
    end

end

fclose(outfile);
