% //////////////////////////////////////////////////////////
% //
% // synthetic test scenes for
% // structured light survey
% // 
% // christoph schmalz
% // 2009
% //
% // christoph(a)structuredlightsurvey.de
% //
% //////////////////////////////////////////////////////////
% //
% // version 1.0 - first public release
% //
% //////////////////////////////////////////////////////////
% //
% // converts the color coded ground truth depthmap
% // into a floating point representation
% //
% // parameters:
% //    -> data = imread(filename)
% // from depthCodeTexture.inc
% //    -> stepsize 20, nsteps 10
% // from testscenes.pov
% //    -> minDepth 500, maxDepth 1500
% //
% //////////////////////////////////////////////////////////

function depth = decodeDepthmap( data )

stepsize = 20;
nsteps = 10;
minDepth = 500;
maxDepth = 1500;

height = size(data,1);
width = size(data,2);

depth = zeros(height,width,3);

for r = 1:height
    for c = 1:width

        %skip black
        if sum(data(r,c,:))==0
            depth(r,c,:) = NaN;
            continue
        end

        %skip white
        if sum(data(r,c,:))==3*255
            depth(r,c,:) = NaN;
            continue
        end

        %red channel contribution in steps of 1/nsteps
        curstep = round(data(r,c,1)/double(stepsize))-1;
        depth(r,c,1) = double(curstep)/double(nsteps);

        %green channel contribution in steps of 1/nsteps^2
        curstep = round(data(r,c,2)/double(stepsize))-1;
        depth(r,c,2) = double(curstep)/double(nsteps^2);
        
        %blue channel contribution must be interpolated
        %interval is 1/nsteps^2
        depth(r,c,3) = double(data(r,c,3))/255./nsteps^2;

    end
end

%add all contributions
depth = sum(depth,3);

%scale
depth = depth*(maxDepth-minDepth);

%shift
depth = depth+minDepth;

