//////////////////////////////////////////////////////////
//
// synthetic test scenes for
// structured light survey
// 
// christoph schmalz
// 2009
//
// christoph(a)structuredlightsurvey.de
//
//////////////////////////////////////////////////////////
//
// version 1.0 - first public release
//
//////////////////////////////////////////////////////////
//
// the projector of the structured light system
//
//////////////////////////////////////////////////////////

//pinhole projector
//per default the projector is located at (0,0,0)
//the light direction is z

#macro projector ( projFocalLength, projImagePlaneSize, aspectRatio, patternImagePng, projColor )
union
{
	//we construct the projector with a unit image plane
	//adjust the focal length to scale
	#local projFocalLengthScaled = projFocalLength/projImagePlaneSize;

	//light source in the projector
	light_source
	{ 
		<0.5,projFocalLengthScaled,0.5>
		rgb projColor
		looks_like {
			sphere { <0,0,0> 0.05 pigment { rgb projColor } } //position relative to light source!
		}
	}

	//projector housing - large enough that the light source is inside
	box
	{
		< 0,-0.01,0 >, < 1, projFocalLengthScaled+0.1, 1 >
		hollow 
		
		clipped_by
		{
			//cut off the bottom - will be replaced by the image to project
			box 
			{
				< -0.1,0,-0.1 >, < 1.1, projFocalLengthScaled+0.2, 1.1 >
			}
		}

		pigment { color rgbt <0.3,0.3,0.6,0> }
		finish { ambient 0.5 diffuse 0.5 }
	}

	//image plane of projector
	plane
	{
		y, 0
		
		texture {
		   pigment {
		     image_map {
		     	png patternImagePng
		     	interpolate 2
		     	once
		     	filter all 1
		      }
		      //need to flip the default orientation of the image map
		      rotate <90,0,0> 
		   }
		}
		clipped_by
		{
			box { < 0,-0.1,0 >, < 1,0.1,1 > }
		}
	}

	//move light source to origin
	translate < -0.5, -projFocalLengthScaled, -0.5 >

	//make it face in positive z-direction
	rotate <-90,0,0>

	//fix aspect ratio
	scale < 1, 1/aspectRatio, 1 >
	
	//make it small
	scale projImagePlaneSize

} //union
#end