//////////////////////////////////////////////////////////
//
// synthetic test scenes for
// structured light survey
// 
// christoph schmalz
// 2009
//
// christoph(a)structuredlightsurvey.de
//
//////////////////////////////////////////////////////////
//
// version 1.0 - first public release
// version 1.1 - added second camera position
//
//////////////////////////////////////////////////////////
//
// core structured light setup of camera and projector
//
//////////////////////////////////////////////////////////

//proj params

#declare projMonoPosition = <0.25,0.01,-1>;
#declare projStereoPosition = <0,0.01,-1>;

//the projector direction is only rotated around the y-axis
//ie there should only be a noteable difference in the x-coord
#declare projTargetPosition = <0,0.01,0>;

#declare projFocalLength = 0.02;
#declare projImagePlaneSize = 0.02;

#declare projColor = 0.5;

//cam params

#declare camMonoPosition = <0,0,-1>;
#declare camStereo1Position = <-0.3,0,-1>;
#declare camStereo2Position = <0.2,0,-1>;

#declare camTargetPosition = <0,0,0>;

#declare camFocalLength = 0.012;
#declare camImagePlaneSize = 0.01;

//to check general setup
#declare camTestPosition = <-0.3,2,-0.8>;
#declare camTestLookAt = <0,0,-0.5>;

#macro structuredLight( aspectRatio, imageFile, camPosition, extraLightColor, TEST  )

	//--------------------------------------------------
	//	projector
	//--------------------------------------------------
	#if (camPosition=0)
		#declare currentProjPosition = projMonoPosition;
	#else
		#declare currentProjPosition = projStereoPosition;
	#end

	#declare currentProjDirection = projTargetPosition - currentProjPosition;
	#include "projector.inc"
	object
	{
		projector(projFocalLength,projImagePlaneSize,aspectRatio,imageFile,projColor)
		rotate <0,degrees(atan(currentProjDirection.x/currentProjDirection.z)),0>
		translate currentProjPosition
	}
	
	//--------------------------------------------------
	//	camera
	//--------------------------------------------------

	#switch (camPosition)
	#case (0)
		#declare currentCamPosition = camMonoPosition;
		#declare currentCamDirection = camTargetPosition-camMonoPosition;
	#break
	#case (1)
		#declare currentCamPosition = camStereo1Position;
		#declare currentCamDirection = camTargetPosition-camStereo1Position;
	#break
	#case (2)
		#declare currentCamPosition = camStereo2Position;
		#declare currentCamDirection = camTargetPosition-camStereo2Position;
	#break
	#end

	#if (TEST=1)
	
		camera 
		{
			look_at camTestLookAt
			location camTestPosition
		}
		
		#if (camPosition=0)
			//marker for mono camera position
			sphere
			{
				camMonoPosition, 0.02
				pigment { rgb < 1, 0, 0.5 > }
				finish { diffuse 0.6 phong 1 }
			}
		#else
			//markers for stereo camera positions
			sphere
			{
				camStereo1Position, 0.02
				pigment { rgb < 1, 0, 0.5 > }
				finish { diffuse 0.6 phong 1 }
			}
			sphere
			{
				camStereo2Position, 0.02
				pigment { rgb < 1, 0, 0.5 > }
				finish { diffuse 0.6 phong 1 }
			}
		#end
	
	#else
	
		camera 
		{
			perspective
	
			right x * camImagePlaneSize
			up y/aspectRatio * camImagePlaneSize
	
			direction z * camFocalLength

			location currentCamPosition
			look_at camTargetPosition

			//no depth of field right now
			/*
		    focal_point camTargetPosition 
		    aperture 0.4
		    confidence 0.99
		    variance 1/256
			*/
		} 
	
	#end

	//some light should be everywhere
	#if (camPosition=0)
		light_source { currentCamPosition rgb extraLightColor shadowless}
	#else
		light_source { camTestPosition rgb extraLightColor shadowless}
	#end


#end