//////////////////////////////////////////////////////////
//
// synthetic test scenes for
// structured light survey
// 
// christoph schmalz
// 2009
//
// christoph(a)structuredlightsurvey.de
//
//////////////////////////////////////////////////////////
//
// version 1.0 - first public release
//
//////////////////////////////////////////////////////////
//
// includes the turbine blade mesh or an impostor object for speed
// taken from http://www.cc.gatech.edu/projects/large_models/
// converted to povray format with poseray
//
// object will be located in a unit box centered at zero
//
//////////////////////////////////////////////////////////

#macro turbineBlade(TEST)

	#if (TEST=1)
		#declare impostor = object {
			box {
				//these numbers come from the header of the .inc below
				<-450.75,-601.212,117.155>,<-98.3788,-3.2735,391.5>
				hollow
			}
		}
		
		#local Min = min_extent ( impostor );
		#local Max = max_extent ( impostor );
		#local Size = Max-Min;
	
		object {
			impostor
			hollow
			translate -Min
			translate -Size/2
			scale 1/max(Size.x,Size.y,Size.z)
			}
		
	#else
	
		#include "blade_POV_geom.inc" //Geometry
	
		#local Min = min_extent ( blade_ );
		#local Max = max_extent ( blade_ );
		#local Size = Max-Min;

		#debug concat( "Min is ", vstr(3,Min,",",0,3), "\n" )
		#debug concat( "Max is ", vstr(3,Max,",",0,3), "\n" )
		#debug concat( "Size is ", vstr(3,Size,",",0,3), "\n" )
	
		//Assembled object that is contained in blade_POV_geom.inc
		object {
	    	blade_
	    	hollow
			translate -Min
			translate -Size/2
			scale 1/max(Size.x,Size.y,Size.z)
			}
	#end

#end